/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported init */

const Clutter = imports.gi.Clutter;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const EdgeDragAction = imports.ui.edgeDragAction;
const Main = imports.ui.main;
const WindowManager = imports.ui.windowManager;
const St = imports.gi.St;
const Gio = imports.gi.Gio;

class Extension {
    constructor() {
        this._settings = ExtensionUtils.getSettings();
        let actions = global.stage.get_actions();

        actions.forEach(a => {
            if (a instanceof WindowManager.AppSwitchAction)
                this._appSwitch = a;
            else if (a instanceof EdgeDragAction.EdgeDragAction &&
                 a._side == St.Side.BOTTOM)
                this._showOsk = a;
            else if (a instanceof EdgeDragAction.EdgeDragAction &&
                 a._side == St.Side.TOP)
                this._unfullscreen = a;
        });

        this._map = [
            { setting: 'overview', action: Main.overview._swipeTracker },
            { setting: 'app-switch', action: this._appSwitch },
            { setting: 'show-osk', action: this._showOsk },
            { setting: 'unfullscreen', action: this._unfullscreen },
            { setting: 'workspace-switch', action: Main.wm._workspaceAnimation._swipeTracker },
        ];

        this._enabledDesc = Object.getOwnPropertyDescriptor(
            Clutter.ActorMeta.prototype, 'enabled');
    }

    _overrideEnabledSetter(obj, set) {
        if (!(obj instanceof Clutter.ActorMeta))
            return;

        const desc = set
            ? { ...this._enabledDesc, set }
            : { ...this._enabledDesc };
        Object.defineProperty(obj, 'enabled', desc);
    }

    enable() {
        const settings = this._settings;

        this._map.forEach(m => {
            settings.bind(m.setting, m.action, 'enabled',
                    Gio.SettingsBindFlags.DEFAULT);

            this._overrideEnabledSetter(m.action, function (value) {
                if (settings.get_boolean(m.setting))
                    this.set_enabled(value);
            });
        });
    }

    disable() {
        this._map.forEach(m => {
            Gio.Settings.unbind(m.action, 'enabled');
            this._overrideEnabledSetter(m.action);
            m.action.enabled = true;
        });
    }
}

function init() {
    return new Extension();
}
